# gnome code assistance common ruby
# Copyright (C) 2013  Jesse van den Kieboom <jessevdk@gnome.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

require 'optparse'

options = {
    :transport => 'dbus',
    :address => ':0'
}

OptionParser.new do |opts|
    opts.on('-t', '--transport TRANSPORT', 'Transport (dbus or http)') do |v|
        options[:transport] = v
    end

    opts.on('-a', '--address ADDRESS', 'Http address') do |v|
        options[:address] = v
    end
end.parse!

require 'gnome/codeassistance/transport/' + options[:transport]
require 'gnome/codeassistance/types'

# vi:ts=4:et
